@@ -20,12 +20,12 @@ no_default = _ID()    # creates a unique object which can be used to detect if a
 # ==============================================================================
 # New in version 0.1.45
 # Support for conditional types, e.g. we can write
-# 
+#
 #  x = config(x, 0.1, Float >= 0., "An 'x' which cannot be negative")
 # ==============================================================================
-    
+
 class _Cast(object):
-    
+
     def __call__( self, value, config_name : str, key_name : str ):
         """ cast 'value' to the proper type """
         raise NotImplementedError("Internal error")
@@ -39,7 +39,7 @@ def _cast_name( cast : type ) -> str:
     if cast is None:
         return ""
     return getattr(cast,"__name__", str(cast))
-    
+
 # ================================
 # Simple wrapper
 # ================================
@@ -57,7 +57,7 @@ class _Simple(_Cast):# NOQA
     def __call__(self, value, config_name : str, key_name : str ):
         """ Cast 'value' to the proper type """
         return self.cast(value) if not self.cast is None else value
-        
+
     def __str__(self) -> str:
         """ Returns readable string """
         return _cast_name(self.cast)
@@ -68,11 +68,11 @@ class _Simple(_Cast):# NOQA
 
 class _Condition(_Cast):
     """ Represents a simple operator condition such as 'Float >= 0.' """
-    
+
     def __init__(self, cast, op, other):
         """ Initialize the condition for a base type 'cast' and an 'op' with an 'other'  """
         self.cast    = cast
-        self.op      = op    
+        self.op      = op
         self.other   = other
         self.l_and   = None
 
@@ -81,9 +81,9 @@ class _Condition(_Cast):
         Combines to conditions with logical AND .
         Requires the left hand 'self' to be a > or >=; the right hand must be < or <=
         This means you can write
-        
+
             x = config("x", 0.5, (Float >= 0.) & (Float < 1.), "Variable x")
-        
+
         """
         if not self.l_and is None:
             raise NotImplementedError("Cannot combine more than two conditions")
@@ -110,10 +110,10 @@ class _Condition(_Cast):
         elif self.op == "lt":
             ok = value < self.other
         else:
-            raise RuntimeError("Internal error: unknown operator %s" % str(self.op))            
+            raise RuntimeError("Internal error: unknown operator %s" % str(self.op))
         _log.verify( ok, "Config '%s': value for key '%s' %s. Found %s", config_name, key_name, self.err_str, value )
         return self.l_and( value, config_name, key_name) if not self.l_and is None else value
-        
+
     def __str__(self) -> str:
         """ Returns readable string """
         s = _cast_name(self.cast) + self._op_str + str(self.other)
@@ -132,7 +132,7 @@ class _Condition(_Cast):
             return "<="
         elif self.op == "lt":
             return "<"
-        raise RuntimeError("Internal error: unknown operator %s" % str(self.op))            
+        raise RuntimeError("Internal error: unknown operator %s" % str(self.op))
 
     @property
     def err_str(self) -> str:
@@ -150,19 +150,19 @@ class _Condition(_Cast):
             else:
                 raise RuntimeError("Internal error: unknown operator %s" % str(cond.op))
             return s
-        
+
         s    = "must " + mk_txt(self)
         if not self.l_and is None:
             s += ", and " + mk_txt(self.l_and)
         return s
 
-             
+
 class _CastCond(_Cast): # NOQA
     """
     Generates compound _Condition's
     See the two members Float and Int
     """
-    
+
     def __init__(self, cast):# NOQA
         self.cast = cast
     def __ge__(self, other) -> bool:# NOQA
@@ -172,7 +172,7 @@ class _CastCond(_Cast): # NOQA
     def __le__(self, other) -> bool:# NOQA
         return _Condition( self.cast, 'le', self.cast(other) )
     def __lt__(self, other) -> bool:# NOQA
-        return _Condition( self.cast, 'lt', self.cast(other) )    
+        return _Condition( self.cast, 'lt', self.cast(other) )
     def __call__(self, value, config_name : str, key_name : str ):
         """ This gets called if the type was used without operators """
         cast = _Simple(self.cast, config_name, key_name)
@@ -180,24 +180,24 @@ class _CastCond(_Cast): # NOQA
     def __str__(self) -> str:
         """ This gets called if the type was used without operators """
         return _cast_name(self.cast)
-        
+
 Float = _CastCond(float)
 Int   = _CastCond(int)
 
 # ================================
 # Enum type for list 'cast's
 # ================================
-        
+
 class _Enum(_Cast):
     """
     Utility class to support enumerator types.
     No need to use this classs directly. It will be automatically instantiated if a list is passed as type, e.g.
-    
+
         code = config("code", "Python", ['C++', 'Python'], "Which language do we love")
-        
-    Note that all list members must be of the same type.        
+
+    Note that all list members must be of the same type.
     """
-        
+
     def __init__(self, enum : list, config_name : str, key_name : str ):
         """ Initializes an enumerator casting type. """
         self.enum = list(enum)
@@ -209,26 +209,26 @@ class _Enum(_Cast):
             except:
                 _log.throw( "Error in config '%s': 'key '%s' members of the list are not of consistent type. Found %s for the first element which does match the type %s of the %ldth element",\
                         config_name, key_name, self.cast, _cast_name(type(self.enum[i])), i )
-        
+
     def __call__( self, value, config_name, key_name ):
         """
         Cast 'value' to the proper type and check is one of the list members
         Raises a KeyError if the value was not found in our enum
         """
         value = self.cast(value, config_name, key_name)
-        _log.verify( value in self.enum, "Config '%s': value for key '%s' %s. Found %s", config_name, key_name, self.err_str, str(value) )        
+        _log.verify( value in self.enum, "Config '%s': value for key '%s' %s. Found %s", config_name, key_name, self.err_str, str(value) )
         return value
 
     @property
     def err_str(self) -> str:
-        """ Nice error string """        
+        """ Nice error string """
         s = "must be one of: '" + str(self.enum[0]) + "'"
         for i in range(1,len(self.enum)-1):
             s += ", '" + str(self.enum[i])
         if len(self.enum) > 1:
             s += " or '" + str(self.enum[-1]) + "'"
         return s
-        
+
     def __str__(self) -> str:
         """ Returns readable string """
         s     = "[ "
@@ -236,11 +236,11 @@ class _Enum(_Cast):
             s += ( ", " + self.enum[i] ) if i > 0 else self.enum[i]
         s += " ]"
         return s
- 
+
 # ================================
 # Multiple types
 # ================================
-        
+
 class _Alt(_Cast):
     """
     Initialize a casting compund "alternative" type, e.g. it the variable may contain several types, each of which is acceptable.
@@ -249,12 +249,12 @@ class _Alt(_Cast):
         config("spread", None, ( None, float ), "Float or None")
         config("spread", 1, ( Int<=-1, Int>=1. ), "A variable which has to be outside (-1,+1)")
     """
-    
+
     def __init__(self, casts : list, config_name : str, key_name : str ):
         """ Initialize a compound cast """
         _log.verify( len(casts) > 0, "Error in config '%s': 'cast' for key '%s' is an empty tuple. Tupeks are used for aleternative types, hence passing empty tuple is not defined", config_name, key_name )
         self.casts = [ _create_caster(cast, config_name, key_name) for cast in casts ]
-    
+
     def __call__( self, value, config_name : str, key_name : str ):
         """ Cast 'value' to the proper type """
         e0   = None
@@ -274,7 +274,7 @@ class _Alt(_Cast):
     def err_str(self):
         """ Returns readable string """
         return "must be one of the following types: " + self.__str__
-        
+
     def __str__(self):
         """ Returns readable string """
         s = ""
@@ -286,7 +286,7 @@ class _Alt(_Cast):
 def _create_caster( cast : type, name : str, key : str ):
     """
     Implements casting.
-    
+
     Parameters
     ----------
         value: value, either from the user or the default value if provided
@@ -300,7 +300,7 @@ def _create_caster( cast : type, name : str, key : str ):
         value : casted value
         __str__ : casting help text. Empty if 'cast' is None
     """
-    
+
     if isinstance(cast, list):
         return _Enum( cast, name, key )
     elif isinstance(cast, tuple):
@@ -314,61 +314,61 @@ def _create_caster( cast : type, name : str, key : str ):
 # Actual Config class
 #
 # ==============================================================================
-    
+
 class Config(OrderedDict):
     """
     A simple Config class.
     Main features
-    
+
     Write
         Set data as usual:
-        
+
             config = Config()
             config['features']  = [ 'time', 'spot' ]
             config['weights']   = [ 1, 2, 3 ]
-            
+
         Use member notation
-        
+
             config.network.samples    = 10000
             config.network.activation = 'relu'
-            
+
     Read
         def read_config( confg ):
-        
+
             features = config("features", [], list )          # reads features and returns a list
             weights  = config("weights", [], np.ndarray )     # reads features and returns a nump array
-            
+
             network  = config.network
             samples  = network('samples', 10000)              # networks samples
             config.done()                                     # returns an error as we haven't read 'network.activitation'
-            
+
     Detaching child configs
     You can also detach a child config, which allows you to store it for later
     use without triggering done() errors for its parent.
-    
+
         def read_config( confg ):
-        
+
             features = config("features", [], list )          # reads features and returns a list
             weights  = config("weights", [], np.ndarray )     # reads features and returns a nump array
-            
+
             network  = config.network.detach()
             samples  = network('samples', 10000)              # networks samples
             config.done()                                     # no error as 'network' was detached
             network.done()                                    # error as network.activation was not read
-    
+
     Self-recording "help"
     When reading a value, specify an optional help text:
-    
+
         def read_config( confg ):
-        
+
             features = config("features", [], list, help="Defines the features" )
             weights  = config("weights", [], np.ndarray, help="Network weights" )
-            
+
             network  = config.network
             samples  = network('samples', 10000, int, help="Number of samples")
             activt   = network('activation', "relu", str, help="Activation function")
             config.done()
-            
+
             config.usage_report()   # prints help
 
     Attributes
@@ -381,7 +381,7 @@ class Config(OrderedDict):
             Converts object into a dict of dict's'
         recorder : dict
             Records usage of the object and its children.
-            
+
     Methods
     -------
         done()
@@ -389,59 +389,70 @@ class Config(OrderedDict):
             If not, a warning will be produced. This helps catching typos.
             The function will the call mark_done() on itself and all children
             such that subsequent calls to done() to not produce error messages
-            
+
         mark_done()
             Marks all elements of this config and all children as 'read' (done).
             Any elements not read explicitly will not be recorded for usage_report
-            
+
         reset_done()
             Removes all usage information for all members of this config and its children.
             Note: this does not affected recorded previous usage.
-            
+
         detach()
             Create a copy of the current object; set it as 'done'; and return it.
             The copy keeps the reference to the usage recorder.
             This is used to defer using children of configs to a later stage.
             See examples above
-            
+
         copy()
             Returns a blank copy of the current config, with a new recorder.
     """
-    
-    def __init__(self, *args, config_name : str = "config", **kwargs):
+
+    def __init__(self, *args, config_name : str = None, **kwargs):
         """
         See help(Config) for a description of this class.
-        
+
         Parameters
         ----------
-            config_name : str, optional
-                Name of the configuration for report_usage
             *args : list
-                List of dictionaries to update() for.    
+                List of dictionaries to update() for.
+                If the first element is a config, and no other parameters
+                are passed, then this object will be copy of that config
+            config_name : str, optional
+                Name of the configuration for report_usage. Default is 'config'
             **kwargs : dict
                 Used to initialize the config, e.g.
                 Config(a=1, b=2)
         """
+        if len(args) == 1 and isinstance(args[0], Config) and config_name is None and len(kwargs) == 0:
+            source               = args[0]
+            self._done           = source._done
+            self._name           = source._name
+            self._children       = source._children
+            self._recorder       = source._recorder
+            OrderedDict.__init__(self, source)
+            return
+
         OrderedDict.__init__(self)
         self._done           = set()
-        self._name           = config_name
+        self._name           = config_name if not config_name is None else "config"
         self._children       = OrderedDict()
         self._recorder       = SortedDict()
         self._recorder._name = self._name
         for k in args:
             self.update(k)
         self.update(kwargs)
-        
+
     @property
     def config_name(self) -> str:
         """ Returns the fully qualified name of this config """
         return self._name
-    
+
     @property
     def children(self) -> OrderedDict:
         """ Returns dictionary of children """
         return self._children
-    
+
     def __str__(self) -> str:
         """ Print myself as dictionary """
         return str(self.as_dict)
@@ -462,14 +473,14 @@ class Config(OrderedDict):
             _log.verify( not n in d, "Cannot convert config to dictionary: found both a regular value, and a child with name '%s'", n)
             d[n] = c
         return d
-        
+
     @property
     def is_empty(self) -> bool:
         """ Checks whether any variables have been set """
         return len(self) + len(self._children) == 0
-    
+
     def done(self, include_children : bool = True, mark_done : bool = True ):
-        """ 
+        """
         Closes the config and checks that no unread parameters remain.
         By default this function also validates that all child configs were done".
 
@@ -477,18 +488,18 @@ class Config(OrderedDict):
             config = Config()
             config.a = 1
             config.child.b = 2
-            
+
             _ = config.a # read a
-            config.done()   # error because confg.child.b has not been read yet        
+            config.done()   # error because confg.child.b has not been read yet
 
         Instead use:
             config = Config()
             config.a = 1
             config.child.b = 2
-            
+
             _ = config.a # read a
             child = config.child.detach()
-            config.done()   # no error, even though confg.child.b has not been read yet        
+            config.done()   # no error, even though confg.child.b has not been read yet
         """
         inputs = set(self)
         rest   = inputs - self._done
@@ -496,22 +507,22 @@ class Config(OrderedDict):
             _log.verify( False, "Error closing '%s': the following config arguments were not read: %s\n\n"\
                                 "Summary of all variables read from this object:\n%s", \
                                         self._name, list(rest), \
-                                        self.usage_report(filter_path=self._name ) )        
+                                        self.usage_report(filter_path=self._name ) )
         if include_children:
             for config in self._children:
                 self._children[config].done(include_children=include_children,mark_done=False)
         if mark_done:
             self.mark_done(include_children=include_children)
         return
-    
+
     def reset_done(self):
         """ Undo done """
         self._done = set()
-    
+
     def mark_done(self, include_children : bool = True ):
         """ Mark all members as being read. Once called calling done() will no longer trigger an error """
         self._done = set( self )
-        if include_children: 
+        if include_children:
             for c in self._childen:
                 c.mark_done(include_children=include_children)
 
@@ -532,56 +543,56 @@ class Config(OrderedDict):
 
     def detach(self,  mark_self_done : bool = True, new_recorder = False ):
         """
-        Creates a copy of the current config, and marks the current config as "done" unless 'mark_self_done' is used.        
+        Creates a copy of the current config, and marks the current config as "done" unless 'mark_self_done' is used.
         The use case for this function is storing sub config's for deferred processing.
-        
+
         Example:
-            
+
             Creates
-            
+
                 config = Config()
                 config['x']        = 1
                 config.child['y']  =2
-                
+
             Using detach
-            
+
                 def g_done_y( config ):
                     y = config("y")
                     config.done()
-                    
+
                 def f_done_x( config ):
                     x = config("x")
                     child = config.child.detach()
                     config.done()   # <-- checks that all members of 'config'
                                     # except 'child' are done
                     g_done_y( child )
-                
+
                 f_done_x( config )
-    
+
         Consistency
         -----------
         By default, the copy will have the same shared recorder as 'self' which means that usage detection
         is shared between the copied and the original config.
         This is to catch mistakes of the following type:
-        
+
             config = Config()
             config.sub.a = 1
-            
+
             _ = config.sub("a", 2)   # read 'a' with default value 2
-            
+
             def f(sub_config):
                 _ = sub_config("a",3) # read 'a' with default value 3
-            f(config.sub)        
-            
+            f(config.sub)
+
         The below will fail with an error message as 'a' is read twice, but with different default values.
-                        
+
         Use new_recorder = True to create a new, clean recorder.
         This is the default when copy() is used.
-        
+
         Parameters
         ----------
-            mark_self_done : bool, optional 
-                If True mark 'self' as 'read'. 
+            mark_self_done : bool, optional
+                If True mark 'self' as 'read'.
                 This allows storing (a copy of) 'self' for deferred processing without triggering a warning when done() is called on a parent object.
             new_recorder : optional
                 False: use recorder of 'self'. That means any usage inconsistencies are detected between the new and old config.
@@ -598,43 +609,43 @@ class Config(OrderedDict):
             new_recorder         = config._recorder
         else:
             config._recorder     = self._recorder
-        
+
         config._children         = { _: self._children[_].detach(mark_self_done=mark_self_done, new_recorder=new_recorder) for _ in self._children }
         config._children         = OrderedDict( config._children )
 
         if mark_self_done:
             self.mark_done()
         return config
-        
+
     def copy( self ):
-        """ 
+        """
         Return a copy of 'self'.
         Do not flag the original config as 'done' but create a new recorder.
         That means that the copied config can be used independently of the 'self'.
-        
-        As an example, this allows using different default values for 
+
+        As an example, this allows using different default values for
         config members of the same name:
-            
+
             base = Config()
             base.a = 1
             copy = base.copy()
 
             _ = base("x", 1)
             _ = copy("x", 2)
-        
+
         This call is equivalent to detach(mark_self_done=False, new_recorder=True)
         """
         return self.detach( mark_self_done=False, new_recorder=True )
-        
+
     def update( self, other=None, **kwargs ):
         """
-        Overwrite values of 'self' new values. 
+        Overwrite values of 'self' new values.
         Accepts the two main formats
-        
+
             update( dictionary )
             update( config )
             update( a=1, b=2 )
-            
+
         Parameters
         ----------
             other : dict, Config, optional
@@ -658,37 +669,37 @@ class Config(OrderedDict):
                     set_recorder(child, self._recorder)
                     self._children[sub]= child
                 # copy elements from other.
-                # we do not mark elements from another config as 'used' 
+                # we do not mark elements from another config as 'used'
                 for key in other:
                     self[key] = OrderedDict.get(other,key)
             else:
                 _log.verify( isinstance(other, Mapping), "Cannot update config with an object of type '%s'. Expected 'Mapping' type.", type(other).__name__ )
                 for key in other:
                     self[key] = other[key]
-                
+
         OrderedDict.update( self, kwargs )
-    
+
     # Read
     # -----
-        
+
     def __call__(self, key          : str,
-                       default      = no_default, 
-                       cast         : type = None, 
-                       help         : str = None, 
-                       help_default : str = None, 
+                       default      = no_default,
+                       cast         : type = None,
+                       help         : str = None,
+                       help_default : str = None,
                        help_cast    : str = None,
                        mark_done    : bool = True,
                        record       : bool = True ):
         """
         Reads 'key' from the config. If not found, return 'default' if specified.
-    
+
             config("key")                      - returns the value for 'key' or if not found raises an exception
             config("key", 1)                   - returns the value for 'key' or if not found returns 1
             config("key", 1, int)              - if 'key' is not found, return 1. If it is found cast the result with int().
             config("key", 1, int, "A number"   - also stores an optional help text.
-                                                 Call usage_report() after the config has been read to a get a full 
+                                                 Call usage_report() after the config has been read to a get a full
                                                  summary of all data requested from this config.
-    
+
         Parameters
         ----------
             key : string
@@ -698,7 +709,7 @@ class Config(OrderedDict):
             cast : object, optional
                 If not None, will attempt to cast the value provided with the provided value.
                 E.g. if cast = int, then it will run int(x)
-                This function now also allows passing a list or tupel, in which case it is assumed that 
+                This function now also allows passing a list or tupel, in which case it is assumed that
                 the 'key' must be from this list.
             help : str, optional
                 If provied adds a help text when self documentation is used.
@@ -713,14 +724,14 @@ class Config(OrderedDict):
             record : bool, optional
                 If True, records usage of the key and validates that previous usage of the key is consistent with
                 the current usage, e.g. that the default values are consistent and that if help was provided it is the same.
-                
+
         Returns
         -------
             Value.
         """
         _log.verify( isinstance(key, str), "'key' must be a string. Found type %s. Details: %s", type(key), key)
         _log.verify( key.find('.') == -1 , "Error in config '%s': key name cannot contain '.'. Found %s", self._name, key )
-        
+
         # determine raw value
         if not key in self:
             if default == no_default:
@@ -732,11 +743,11 @@ class Config(OrderedDict):
         # casting
         caster = _create_caster( cast, self._name, key )
         value  = caster( value, self._name, key )
-                        
+
         # mark key as read, and record call
         if mark_done:
             self._done.add(key)
-        
+
         # avoid recording
         if not record:
             return value
@@ -749,7 +760,7 @@ class Config(OrderedDict):
         help_default  = str(default) if default != no_default and len(help_default) == 0 else help_default
         help_cast     = help_cast if not help_cast is None else str(caster)
         _log.verify( default != no_default or help_default == "", "Config %s setup error for key %s: cannot specify 'help_default' if no default is given", self._name, key )
-        
+
         raw_use       = help == "" and help_cast == "" and help_default == "" # raw_use, e.g. simply get() or []. Including internal use
         record        = SortedDict( value=value,
                                     raw_use=raw_use,
@@ -758,7 +769,7 @@ class Config(OrderedDict):
                                     help_cast=help_cast )
         if default != no_default:
             record['default'] = default
-                
+
         exst_value    = self._recorder.get(record_key, None)
 
         if exst_value is None:
@@ -772,12 +783,12 @@ class Config(OrderedDict):
                 else:
                     # Both current and past were bona fide uses. Ensure that their usage is consistent.
                     _log.verify( exst_value['default'] == default,  "Config %s was read twice for key %s (%s) with different default values %s and %s", self._name, key, record_key, exst_value['default'], default )
-                    _log.verify( exst_value['help_default'] == help_default, "Config %s was read twice for key %s (%s) with different 'help_default' values %s and %s", self._name, key, record_key, exst_value['help_default'], help_default )                    
+                    _log.verify( exst_value['help_default'] == help_default, "Config %s was read twice for key %s (%s) with different 'help_default' values %s and %s", self._name, key, record_key, exst_value['help_default'], help_default )
                     _log.verify( exst_value['help'] == help, "Config %s was read twice for key %s (%s) with different 'help' values %s and %s", self._name, key, record_key, exst_value['help'], help )
                     _log.verify( exst_value['help_cast'] == help_cast, "Config %s was read twice for key %s (%s) with different 'help_cast' values %s and %s", self._name, key, record_key, exst_value['help_cast'], help_cast )
         # done
         return value
-        
+
     def __getitem__(self, key : str):
         """ Returns self(key) """
         return self(key)
@@ -815,10 +826,10 @@ class Config(OrderedDict):
         if record is None:
             raise KeyError(key)
         return record['value']
-    
+
     # Write
     # -----
-    
+
     def __setattr__(self, key, value):
         """
         Assign value like self[key] = value
@@ -836,12 +847,12 @@ class Config(OrderedDict):
 
     # Recorder
     # --------
-    
+
     @property
     def recorder(self) -> SortedDict:
         """ Returns the top level recorder """
         return self._recorder
-    
+
     def usage_report(self,    with_values  : bool = True,
                               with_help    : bool = True,
                               with_defaults: bool = True,
@@ -849,26 +860,26 @@ class Config(OrderedDict):
                               filter_path  : str  = None ) -> str:
         """
         Generate a human readable report of all variables read from this config.
-    
+
         Parameters
         ----------
             with_values : bool, optional
                 Whether to also print values. This can be hard to read
                 if values are complex objects
-                
+
             with_help: bool, optional
                 Whether to print help
-                
+
             with_defaults: bool, optional
                 Whether to print default values
-                
+
             with_cast: bool, optional
                 Whether to print types
-                
+
             filter_path : str, optional
                 If provided, will match all children names vs this string.
                 Most useful with filter_path = self._name
-                
+
         Returns
         -------
             str
@@ -890,10 +901,10 @@ class Config(OrderedDict):
             help_cast    =  record['help_cast']
             report       =  key + " = " + str(value) if with_values else key
 
-            do_help      =  with_help and help != ""      
+            do_help      =  with_help and help != ""
             do_cast      =  with_cast and help_cast != ""
             do_defaults  =  with_defaults and help_default != ""
-            
+
             if do_help or do_cast or do_defaults:
                 report += " # "
                 if do_cast:
@@ -904,8 +915,8 @@ class Config(OrderedDict):
                         report += "; default: " + help_default
                 elif do_defaults:
                     report += " Default: " + help_default
-                    
-            if l > 0 and key[:l] == filter_path: 
+
+            if l > 0 and key[:l] == filter_path:
                 rep_here += report + "\n"
             else:
                 reported += report + "\n"
@@ -928,13 +939,13 @@ class Config(OrderedDict):
             value        =  record['value']
             report       += key + " = " + repr(value) + "\n"
         return report
-    
+
     def input_report(self) -> str:
         """
         Returns a report of all inputs in a readable format, as long as all values
         are as such.
         """
-        inputs = []      
+        inputs = []
         def ireport(self, inputs):
             for key in self:
                 value      = self.get_raw(key)
@@ -943,16 +954,16 @@ class Config(OrderedDict):
             for c in self._children:
                 ireport(self._children[c], inputs)
         ireport(self, inputs)
-        
+
         inputs = sorted(inputs)
         report = ""
         for i in inputs:
-            report += i + "\n"        
+            report += i + "\n"
         return report
-        
+
     def input_dict(self, ignore_underscore = True ) -> dict:
         """ Returns a (pretty) dictionary of all inputs into this config. """
-        inputs = pdct()  
+        inputs = pdct()
         for key in self:
             if ignore_underscore and key[:1] == "_":
                 continue
@@ -962,15 +973,15 @@ class Config(OrderedDict):
                 continue
             inputs[c] = self._children[c].input_dict()
         return inputs
-        
+
     def unique_id(self, length = None, parse_functions = False ) -> str:
         """
         Returns an MDH5 hash key for this object, based on its provided inputs /not/ based on its usage
         ** WARNING **
-        This function ignores 
+        This function ignores
          1) Config keys or children with leading '_'s
          2) Functions and properties unless parse_functions is True
-            In the latter case function code will be used to distinguish 
+            In the latter case function code will be used to distinguish
             functions assigned to the config.
         See util.unqiueHashExt() for further information.
         """
@@ -978,12 +989,12 @@ class Config(OrderedDict):
         for key in self:
             if key[:1] == "_":
                 continue
-            inputs[key] = self.get_raw(key) 
+            inputs[key] = self.get_raw(key)
         for c in self._children:
             if c[:1] == "_":
                 continue
             # collect ID for the child
-            child_data = self._children[c].unique_id() 
+            child_data = self._children[c].unique_id()
             # we only register children if they have keys.
             # this way we do not trigger a change in ID simply due to a failed read access.
             if child_data != "":
@@ -992,17 +1003,17 @@ class Config(OrderedDict):
 
     # magic
     # -----
-    
+
     def __iter__(self):
         """
-        Iterate. For some odd reason, adding this override will make 
+        Iterate. For some odd reason, adding this override will make
         using f(**self) call our __getitem__() function.
         """
         return OrderedDict.__iter__(self)
-    
+
     # pickling
     # --------
-    
+
     def __reduce__(self):
         """
         Pickling this object explicitly
@@ -1017,7 +1028,7 @@ class Config(OrderedDict):
                      keys = keys,
                      data = data )
         return (Config, (), state)
-    
+
     def __setstate__(self, state):
         """ Supports unpickling """
         self._name = state['name']
@@ -1029,9 +1040,16 @@ class Config(OrderedDict):
         for (k,d) in zip(keys,data):
             self[k] = d
 
+def to_config( kwargs, config_name = "kwargs"):
+    """
+    Makes sure an object is a config, and otherwise tries to convert it into one
+    Classic use case is to transform 'kwargs' to a Config
+    """
+    return kwargs if isinstance(kwargs,Config) else Config( kwargs,config_name=config_name )
+
 def __test_pickle():
     import pickle
-    
+
     config = Config()
     # world
     config.world.samples = 10000
@@ -1042,7 +1060,7 @@ def __test_pickle():
     config.world.cost_s = 0.
     # gym
     config.gym.objective.utility = "cvar"
-    config.gym.objective.lmbda = 1.  
+    config.gym.objective.lmbda = 1.
     config.gym.agent.network.depth = 5   # <====== changed this
     config.gym.agent.network.width = 40
     config.gym.agent.network.activation = "softplus"
@@ -1056,15 +1074,15 @@ def __test_pickle():
     config.trainer.visual.time_refresh = 10
     config.trainer.visual.confidence_pcnt_lo = 0.25
     config.trainer.visual.confidence_pcnt_hi = 0.75
-    
+
     id2 = config.unique_id()
-    
+
     # pickle test
-    
+
     binary   = pickle.dumps(config)
     restored = pickle.loads(binary)
     idrest   = restored.unique_id()
     assert idrest == id2, (idrest, id2)
-    
-        
-        
+
+
+
